@@ -1,4 +1,5 @@
 /* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Init.c,v 3.212 2004/01/27 01:31:45 dawes Exp $ */
+/* $XdotOrg: $ */
 
 /*
  * Loosely based on code bearing the following copyright:
@@ -1137,7 +1138,8 @@ InitInput(argc, argv)
     else {
 #ifdef USE_DEPRECATED_KEYBOARD_DRIVER
       /* Only set this if we're allowing the old driver. */
-      xf86Info.pKeyboard = AddInputDevice(xf86Info.kbdProc, TRUE);
+	if (xf86Info.kbdProc != NULL) 
+	    xf86Info.pKeyboard = AddInputDevice(xf86Info.kbdProc, TRUE);
 #endif
     }
     if (corePointer)
@@ -1366,14 +1368,20 @@ ddxProcessArgument(int argc, char **argv, int i)
    * yet.  Use malloc/free instead.
    */
 
+#define CHECK_FOR_REQUIRED_ARGUMENT() \
+    if (((i + 1) >= argc) || (!argv[i + 1])) { 				\
+      ErrorF("Required argument to %s not specified\n", argv[i]); 	\
+      UseMsg(); 							\
+      FatalError("Required argument to %s not specified\n", argv[i]);	\
+    }
+  
   /* First the options that are only allowed for root */
   if (getuid() == 0 || geteuid != 0)
   {
     if (!strcmp(argv[i], "-modulepath"))
     {
       char *mp;
-      if (!argv[i + 1])
-	return 0;
+      CHECK_FOR_REQUIRED_ARGUMENT();
       mp = malloc(strlen(argv[i + 1]) + 1);
       if (!mp)
 	FatalError("Can't allocate memory for ModulePath\n");
@@ -1385,8 +1393,7 @@ ddxProcessArgument(int argc, char **argv, int i)
     else if (!strcmp(argv[i], "-logfile"))
     {
       char *lf;
-      if (!argv[i + 1])
-	return 0;
+      CHECK_FOR_REQUIRED_ARGUMENT();
       lf = malloc(strlen(argv[i + 1]) + 1);
       if (!lf)
 	FatalError("Can't allocate memory for LogFile\n");
@@ -1395,16 +1402,18 @@ ddxProcessArgument(int argc, char **argv, int i)
       xf86LogFileFrom = X_CMDLINE;
       return 2;
     }
+  } else if (!strcmp(argv[i], "-modulepath") || !strcmp(argv[i], "-logfile")) {
+    FatalError("The '%s' option can only be used by root.\n", argv[i]);
   }
   if (!strcmp(argv[i], "-config") || !strcmp(argv[i], "-xf86config"))
   {
-    if (!argv[i + 1])
-      return 0;
+    CHECK_FOR_REQUIRED_ARGUMENT();
     if (getuid() != 0 && !xf86PathIsSafe(argv[i + 1])) {
-      FatalError("\nInvalid argument for -config\n"
-	  "\tFor non-root users, the file specified with -config must be\n"
+      FatalError("\nInvalid argument for %s\n"
+	  "\tFor non-root users, the file specified with %s must be\n"
 	  "\ta relative path and must not contain any \"..\" elements.\n"
-	  "\tUsing default "__XCONFIGFILE__" search path.\n\n");
+	  "\tUsing default "__XCONFIGFILE__" search path.\n\n",
+	  argv[i], argv[i]);
     }
     xf86ConfigFile = argv[i + 1];
     return 2;
@@ -1539,11 +1548,11 @@ ddxProcessArgument(int argc, char **argv, int i)
   }
   if (!strcmp(argv[i], "-bpp"))
   {
-    if (++i >= argc)
-      return 0;
     ErrorF("The -bpp option is no longer supported.\n"
 	"\tUse -depth to set the color depth, and use -fbbpp if you really\n"
 	"\tneed to force a non-default framebuffer (hardware) pixel format.\n");
+    if (++i >= argc)
+      return 1;
     return 2;
   }
   if (!strcmp(argv[i], "-pixmap24"))
@@ -1559,9 +1568,8 @@ ddxProcessArgument(int argc, char **argv, int i)
   if (!strcmp(argv[i], "-fbbpp"))
   {
     int bpp;
-    if (++i >= argc)
-      return 0;
-    if (sscanf(argv[i], "%d", &bpp) == 1)
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    if (sscanf(argv[++i], "%d", &bpp) == 1)
     {
       xf86FbBpp = bpp;
       return 2;
@@ -1575,9 +1583,8 @@ ddxProcessArgument(int argc, char **argv, int i)
   if (!strcmp(argv[i], "-depth"))
   {
     int depth;
-    if (++i >= argc)
-      return 0;
-    if (sscanf(argv[i], "%d", &depth) == 1)
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    if (sscanf(argv[++i], "%d", &depth) == 1)
     {
       xf86Depth = depth;
       return 2;
@@ -1591,9 +1598,8 @@ ddxProcessArgument(int argc, char **argv, int i)
   if (!strcmp(argv[i], "-weight"))
   {
     int red, green, blue;
-    if (++i >= argc)
-      return 0;
-    if (sscanf(argv[i], "%1d%1d%1d", &red, &green, &blue) == 3)
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    if (sscanf(argv[++i], "%1d%1d%1d", &red, &green, &blue) == 3)
     {
       xf86Weight.red = red;
       xf86Weight.green = green;
@@ -1610,9 +1616,8 @@ ddxProcessArgument(int argc, char **argv, int i)
       !strcmp(argv[i], "-ggamma") || !strcmp(argv[i], "-bgamma"))
   {
     double gamma;
-    if (++i >= argc)
-      return 0;
-    if (sscanf(argv[i], "%lf", &gamma) == 1) {
+    CHECK_FOR_REQUIRED_ARGUMENT();    
+    if (sscanf(argv[++i], "%lf", &gamma) == 1) {
        if (gamma < GAMMA_MIN || gamma > GAMMA_MAX) {
 	  ErrorF("gamma out of range, only  %.2f <= gamma_value <= %.1f"
 		 " is valid\n", GAMMA_MIN, GAMMA_MAX);
@@ -1628,30 +1633,26 @@ ddxProcessArgument(int argc, char **argv, int i)
   }
   if (!strcmp(argv[i], "-layout"))
   {
-    if (++i >= argc)
-      return 0;
-    xf86LayoutName = argv[i];
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    xf86LayoutName = argv[++i];
     return 2;
   }
   if (!strcmp(argv[i], "-screen"))
   {
-    if (++i >= argc)
-      return 0;
-    xf86ScreenName = argv[i];
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    xf86ScreenName = argv[++i];
     return 2;
   }
   if (!strcmp(argv[i], "-pointer"))
   {
-    if (++i >= argc)
-      return 0;
-    xf86PointerName = argv[i];
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    xf86PointerName = argv[++i];
     return 2;
   }
   if (!strcmp(argv[i], "-keyboard"))
   {
-    if (++i >= argc)
-      return 0;
-    xf86KeyboardName = argv[i];
+    CHECK_FOR_REQUIRED_ARGUMENT();    
+    xf86KeyboardName = argv[++i];
     return 2;
   }
   if (!strcmp(argv[i], "-nosilk"))
@@ -1684,11 +1685,9 @@ ddxProcessArgument(int argc, char **argv, int i)
   if (!strcmp(argv[i], "-isolateDevice"))
   {
     int bus, device, func;
-    if (++i >= argc)
-       return 0;
-    if (strncmp(argv[i], "PCI:", 4)) {
-       ErrorF("Bus types other than PCI not yet isolable\n");
-       return 0;
+    CHECK_FOR_REQUIRED_ARGUMENT();
+    if (strncmp(argv[++i], "PCI:", 4)) {
+       FatalError("Bus types other than PCI not yet isolable\n");
     }
     if (sscanf(argv[i], "PCI:%d:%d:%d", &bus, &device, &func) == 3) {
        xf86IsolateDevice.bus = bus;
@@ -1696,8 +1695,7 @@ ddxProcessArgument(int argc, char **argv, int i)
        xf86IsolateDevice.func = func;
        return 2;
     } else {
-       ErrorF("Invalid isolated device specifiation\n");
-       return 0;
+       FatalError("Invalid isolated device specification\n");
     }
   }
   /* OS-specific processing */
@@ -1727,11 +1725,8 @@ ddxUseMsg()
     ErrorF("-logfile file          specify a log file name\n");
     ErrorF("-configure             probe for devices and write an "__XCONFIGFILE__"\n");
   }
-  else
-  {
-    ErrorF("-config file       specify a configuration file, relative to the\n");
-    ErrorF("                       "__XCONFIGFILE__" search path, only root can use absolute\n");
-  }
+  ErrorF("-config file           specify a configuration file, relative to the\n");
+  ErrorF("                       "__XCONFIGFILE__" search path, only root can use absolute\n");
   ErrorF("-probeonly             probe for devices, then exit\n");
   ErrorF("-scanpci               execute the scanpci module and exit\n");
   ErrorF("-verbose [n]           verbose startup messages\n");
@@ -1804,11 +1799,21 @@ xf86PrintBanner()
 #endif
 
 #if XORG_VERSION_SNAP >= 900
+  /* When the minor number is 99, that signifies that the we are making
+   * a release candidate for a major version.  (X.0.0)
+   * When the patch number is 99, that signifies that the we are making
+   * a release candidate for a minor version.  (X.Y.0)
+   * When the patch number is < 99, then we are making a release
+   * candidate for the next point release.  (X.Y.Z)
+   */
 #if XORG_VERSION_MINOR >= 99
   ErrorF(" (%d.0.0 RC %d)", XORG_VERSION_MAJOR+1, XORG_VERSION_SNAP - 900);
-#else
+#elif XORG_VERSION_PATCH == 99
   ErrorF(" (%d.%d.0 RC %d)", XORG_VERSION_MAJOR, XORG_VERSION_MINOR + 1,
 				XORG_VERSION_SNAP - 900);
+#else
+  ErrorF(" (%d.%d.%d RC %d)", XORG_VERSION_MAJOR, XORG_VERSION_MINOR,
+ 			 XORG_VERSION_PATCH + 1, XORG_VERSION_SNAP - 900);
 #endif
 #endif
 
