@@ -48,10 +48,6 @@ SOFTWARE.
 /* $Xorg: servermd.h,v 1.3 2000/08/17 19:53:31 cpqbld Exp $ */
 /* $XdotOrg: xc/programs/Xserver/include/servermd.h,v 1.4 2005/05/21 07:46:38 alanc Exp $ */
 
-#ifdef HAVE_DIX_CONFIG_H
-#include <dix-config.h>
-#endif
-
 #ifndef SERVERMD_H
 #define SERVERMD_H 1
 
@@ -215,10 +211,16 @@ SOFTWARE.
 #endif /* SuperH */
 
 
-#if (defined(sun) && !(defined(i386) && defined(SVR4))) || \
+#if (defined(sun) && (defined(__sparc) || defined(sparc))) || \
     (defined(__uxp__) && (defined(sparc) || defined(mc68000))) || \
     defined(__sparc__) || defined(__mc68000__)
 
+#if defined(__sparc) || defined(__sparc__)
+# if !defined(sparc)
+#  define sparc 1
+# endif
+#endif
+
 #if defined(sun386) || defined(sun5)
 # define IMAGE_BYTE_ORDER	LSBFirst        /* Values for the SUN only */
 # define BITMAP_BIT_ORDER	LSBFirst
@@ -447,7 +449,7 @@ SOFTWARE.
 
 #if	(defined(SVR4) && defined(i386)) || \
 	defined(__alpha__) || defined(__alpha) || \
-	defined(__i386__) || \
+	defined(__i386__) || defined(__i386) || \
 	defined(__UNIXOS2__) || \
 	defined(__OS2ELF__) || \
 	defined(__QNX__) || \
